/*
 * Generic Hash and HMAC Program
 *
 * Copyright (C) 2009 2011 2016 Harald von Fellenberg <hvf@hvf.ch>
 *
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 3 of the License, or 
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _EXTRA_INTERFACE_H_
#define _EXTRA_INTERFACE_H_
#

typedef uint8_t BitSequence;

typedef size_t BitLength;


/* this is how we pass extra data to the XOF functions	*/
typedef struct	extra {

    /* magic token - SHA3-xxx in LSB notation   */
    DataLength      magic;

	/* base64 output flag	*/
	int	base64flag;

	/* extendable output function flag (XOF) and length
	 * if set, xoflength gives the requested length in bytes
	 * if cleared, default length is produced */
	int	xofflag;

	/* -1 means infinity, default is 512 bytes, HASH_LENGTH_SHAKExxx	*/
	long long	xoflength;
    BitLength   xofBitLen;

	/* binary output flag (not honored for output on terminal)	*/
	int	binoutflag;

	/* return parameters: this_chunk_size, more_size
	 * this_chunk_size tells how much data we get in this call to xxx_Final
	 * more_size tells how much data is still waiting after this call
	 * more_size == 0 means we are done
	 */

	size_t	this_chunk_size;
	long long	more_size;
	
    /* KMAC additional parameters   */
    BitSequence *key;
    BitLength   KeyBitLen;
    BitLength   outputBitLen;
    BitLength   digestBitLen;
    BitSequence *customize;
    BitLength   customizeBitLen;

#ifndef BLAKE3_KEY_LEN
#define BLAKE3_KEY_LEN 32
#endif

    /* BLAKE3 additional parameters */
    /* length of input hexstring in bytes   */
    int         b3byteflag;
    int         b3bytecount;

    /* BLAKE3 context in key derivation mode, a null-terminated ASCII string */
    int         b3contextflag;
    char        *b3context;

    /* BLAKE3 key for keyed hashing mode. The key must be exactly 32 bytes */
    int         b3keyflag;
    uint8_t     b3key[BLAKE3_KEY_LEN];  // ASCII, must be 32 bytes long

} *EXTRA_DATA;
	
#endif
